package com.astrum.io;

import java.io.IOException;

public abstract class IOGPIO extends IONative {
	int type;
	String gpio;
	private boolean nonc = false;

	protected IOGPIO(int type, String gpio) {
		this.type = type;
		this.gpio = gpio;
	}

	public String getGpioName() {
		return gpio;
	}

	@Override
	protected boolean IOOpen() throws IOException {
		nativeId = nativeCreate(type, gpio);
		boolean result = nativeOpen(nativeId);
		if (!result) {
			this.release();
			IOLog.logE(gpio + " not opened!");
		} else {
			IOLog.log(gpio + " opened");
		}
		return result;
	}

	@Override
	protected boolean IOClose() throws IOException {
		IOLog.log(gpio + " closed");
		this.release();
		return true;
	}

	public void release() throws IOException {
		nativeRelease(nativeId);
	}

	public boolean getNonc() {
		return nonc;
	}

	public void setNonc(boolean nonc) {
		this.nonc = nonc;
	}

	private native long nativeCreate(int type, String port);

	private native void nativeRelease(long nativeId);

	private native boolean nativeOpen(long nativeId);

	private native boolean nativeClose(long nativeId);

	protected native boolean nativeGet(long nativeId);

	protected native void nativePut(long nativeId, boolean value);
}
