package com.astrum.sample;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.NavUtils;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.CheckBox;

import com.astrum.io.IOGpioInput;
import com.astrum.io.IOGpioOutput;
import com.astrum.sdk.R;

import java.io.IOException;
import java.util.concurrent.locks.ReentrantLock;

public class GPIOActivity extends Activity {
    CheckBox chkInput1;
    CheckBox chkInput2;
    CheckBox chkInput3;
    CheckBox chkInput4;
    CheckBox chkInput5;
    CheckBox chkOutput1;
    CheckBox chkOutput2;
    CheckBox chkExternalOutput1;
    CheckBox chkExternalOutput2;
    IOGpioInput input1;
    IOGpioInput input2;
    IOGpioInput input3;
    IOGpioInput input4;
    IOGpioInput input5;

    IOGpioOutput output1;
    IOGpioOutput output2;
    IOGpioOutput externalOutput1;
    IOGpioOutput externalOutput2;

    Handler uiHandler = new Handler();
    ReentrantLock outputLock=new ReentrantLock();
    boolean isResumed=false;
    GPIOThread currentGpioThread=null;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_gpio);
        chkInput1 = (CheckBox) findViewById(R.id.chkInput1);
        chkInput2 = (CheckBox) findViewById(R.id.chkInput2);
        chkInput3 = (CheckBox) findViewById(R.id.chkInput3);
        chkInput4 = (CheckBox) findViewById(R.id.chkInput4);
        chkInput5 = (CheckBox) findViewById(R.id.chkInput5);
        chkOutput1 = (CheckBox) findViewById(R.id.chkOutput1);
        chkOutput2 = (CheckBox) findViewById(R.id.chkOutput2);
        chkExternalOutput1 = (CheckBox) findViewById(R.id.chkExternalOutput1);
        chkExternalOutput2 = (CheckBox) findViewById(R.id.chkExternalOutput2);
        View.OnClickListener outputClickListener=new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    outputLock.lock();
                    if(v.getId()==chkOutput1.getId()){
                        output1.put(chkOutput1.isChecked());
                    }else if(v.getId()==chkOutput2.getId()){
                        output2.put(chkOutput2.isChecked());
                    }else if(v.getId()==chkExternalOutput1.getId()){
                        externalOutput1.put(chkExternalOutput1.isChecked());
                    }else if(v.getId()==chkExternalOutput2.getId()){
                        externalOutput2.put(chkExternalOutput2.isChecked());
                    }
                }catch (Exception e){
                    e.printStackTrace();
                }finally {
                    outputLock.unlock();
                }
            }
        };
        chkOutput1.setOnClickListener(outputClickListener);
        chkOutput2.setOnClickListener(outputClickListener);
        chkExternalOutput1.setOnClickListener(outputClickListener);
        chkExternalOutput2.setOnClickListener(outputClickListener);
    }

    @Override
    protected void onResume() {
        super.onResume();
        getActionBar().setDisplayHomeAsUpEnabled(true);
        isResumed=true;
        if(currentGpioThread!=null){
            isResumed=false;
            try {
                currentGpioThread.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        isResumed=true;
        currentGpioThread=new GPIOThread();
        currentGpioThread.start();
    }

    @Override
    protected void onPause() {
        super.onPause();
        isResumed=false;
        try {
            currentGpioThread.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                NavUtils.navigateUpFromSameTask(this);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }
    class GPIOThread extends Thread {
        @Override
        public void run() {
            try {
                input1 = new IOGpioInput(IOGpioInput.IOInputNames.INPUT_1);
                input1.open();
                input2 = new IOGpioInput(IOGpioInput.IOInputNames.INPUT_2);
                input2.open();
                input3 = new IOGpioInput(IOGpioInput.IOInputNames.INPUT_3);
                input3.open();
                input4 = new IOGpioInput(IOGpioInput.IOInputNames.INPUT_4);
                input4.open();
                input5 = new IOGpioInput(IOGpioInput.IOInputNames.INPUT_5);
                input5.open();
                output1 = new IOGpioOutput(IOGpioOutput.IOOutputNames.OUTPUT_1);
                output1.open();
                output2 = new IOGpioOutput(IOGpioOutput.IOOutputNames.OUTPUT_2);
                output2.open();
                externalOutput1 = new IOGpioOutput(IOGpioOutput.IOOutputNames.EXTERNAL_OUTPUT_1);
                externalOutput1.open();
                externalOutput2 = new IOGpioOutput(IOGpioOutput.IOOutputNames.EXTERNAL_OUTPUT_2);
                externalOutput2.open();
                while (isResumed) {
                    boolean isChanged = chkInput1.isChecked() != input1.get() ||
                            chkInput2.isChecked() != input2.get() ||
                            chkInput3.isChecked() != input3.get() ||
                            chkInput4.isChecked() != input4.get() ||
                            chkInput5.isChecked() != input5.get() ||
                            chkOutput1.isChecked() != output1.get() ||
                            chkOutput2.isChecked() != output2.get() ||
                            chkExternalOutput1.isChecked() != externalOutput1.get() ||
                            chkExternalOutput2.isChecked() != externalOutput2.get();

                        if(isChanged){
                            uiHandler.post(new Runnable() {
                                @Override
                                public void run() {
                                    try{
                                        outputLock.lock();
                                        chkInput1.setChecked(input1.get());
                                        chkInput2.setChecked(input2.get());
                                        chkInput3.setChecked(input3.get());
                                        chkInput4.setChecked(input4.get());
                                        chkInput5.setChecked(input5.get());
                                        chkOutput1.setChecked(output1.get());
                                        chkOutput2.setChecked(output2.get());
                                        chkExternalOutput1.setChecked(externalOutput1.get());
                                        chkExternalOutput2.setChecked(externalOutput2.get());
                                    }catch (Exception e) {
                                        e.printStackTrace();
                                    }finally {
                                        outputLock.unlock();
                                    }
                                }
                            });
                        }
                    Thread.sleep(50);
                }
            } catch (Exception e) {
                e.printStackTrace();

            } finally {
                Log.d("GPIO","closed");
                try {
                    input1.close();
                    input2.close();
                    input3.close();
                    input4.close();
                    input5.close();
                    output1.close();
                    output2.close();
                    externalOutput1.close();
                    externalOutput2.close();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }

            }

        }
    }
}
